==> kite-1.0b6.tar.gz: kite-1.0b6/modules/__internal/_cinvoke_library.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "config.h"

#ifdef BUILD_CINVOKE
#include "cinvoke/cinvoke.h"

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_construct) {
    kite_object_t *ctxObj, *libString;
    CInvContext *ctx;
    char *lib;
    
    (void)thd;
    
    KITE_GET_METHOD_ARGUMENT(ctxObj, 1);
    KITE_GET_METHOD_ARGUMENT(libString, 2);
    
    ctx = (CInvContext*)ctxObj->builtin_data.filevalue;
    lib = KITE_GET_STRING_VALUE(libString);
    
    this->builtin_data.listvalue.car = 
        (kite_object_t*)ctx;
    this->builtin_data.listvalue.cdr =
        (kite_object_t*)cinv_library_create(ctx, lib);
}

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_destruct) {
    KITE_NO_ARGS;
    (void)thd;
    
    CInvContext *ctx = (CInvContext*)this->builtin_data.listvalue.car;
    CInvLibrary *lib = (CInvLibrary*)this->builtin_data.listvalue.cdr;
    
    cinv_library_delete(ctx, lib);
}

/*****************************************************************************
 * Get function entrypoint
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_entryPoint) {
    kite_object_t *name;
    
    KITE_GET_METHOD_ARGUMENT(name, 1);
    
    CInvContext *ctx = (CInvContext*)this->builtin_data.listvalue.car;
    CInvLibrary *lib = (CInvLibrary*)this->builtin_data.listvalue.cdr;
    
    kite_vm_return(thd, 
        kite_new_integer(thd, 
            (long)cinv_library_load_entrypoint(
                ctx, 
                lib, 
                KITE_GET_STRING_VALUE(name)
            )
        )
    );
}
#endif

KITE_MODULE_INITIALIZER(_cinvoke_library) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_cinvoke_library");
    kite_set_docstring(newclass, "CInvokeLibrary internal code.");

#ifdef BUILD_CINVOKE    
    /* Add methods. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_with_docs(thread, FUNC_COMPILED, CInvokeLibrary_construct, 
                                  "Class constructor.", 2,
                                  "ctx", "CInvokeContext instance",
                                  "lib", "Path to library to load"));
    kite_add_method(thread, newclass, "destruct",
        kite_new_method_with_docs(thread, FUNC_COMPILED, CInvokeLibrary_destruct, 
                                  "Free associated memory and resources.", 0));
    kite_add_method(thread, newclass, "get_entrypoint",
        kite_new_method_with_docs(thread, FUNC_COMPILED, CInvokeLibrary_entryPoint, 
                                  "Retrieves function pointer for given function.", 1,
                                  "name", "Name of function."));
#endif

    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_cinvoke_library", 1, NULL);
    kite_set_property(thread, parent, "_cinvoke_library", newclass);
}

==> kite_1.0.2.orig.tar.gz: kite-1.0.2/modules/__internal/_cinvoke_library.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "config.h"

#ifdef BUILD_CINVOKE
#include "cinvoke/cinvoke.h"

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_construct) {
    kite_object_t *ctxObj, *libString;
    CInvContext *ctx;
    char *lib;
    
    (void)thd;
    
    KITE_GET_METHOD_ARGUMENT(ctxObj, 1);
    KITE_GET_METHOD_ARGUMENT(libString, 2);
    
    ctx = (CInvContext*)ctxObj->builtin_data.filevalue;
    if (libString->type == OBJ_NULL)
        lib = NULL;
    else lib = KITE_GET_STRING_VALUE(libString);
    
    this->builtin_data.listvalue.car = 
        (kite_object_t*)ctx;
    this->builtin_data.listvalue.cdr =
        (kite_object_t*)cinv_library_create(ctx, lib);
}

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_destruct) {
    CInvContext *ctx = (CInvContext*)this->builtin_data.listvalue.car;
    CInvLibrary *lib = (CInvLibrary*)this->builtin_data.listvalue.cdr;

    KITE_NO_ARGS;
    (void)thd;
    
    cinv_library_delete(ctx, lib);
}

/*****************************************************************************
 * Get function entrypoint
 ****************************************************************************/
KITE_CLASS_METHOD(CInvokeLibrary_entryPoint) {
    kite_object_t *name;
    CInvContext *ctx = (CInvContext*)this->builtin_data.listvalue.car;
    CInvLibrary *lib = (CInvLibrary*)this->builtin_data.listvalue.cdr;
    
    KITE_GET_METHOD_ARGUMENT(name, 1);
    
    kite_vm_return(thd, 
        kite_new_integer(thd, 
            (long)cinv_library_load_entrypoint(
                ctx, 
                lib, 
                KITE_GET_STRING_VALUE(name)
            )
        )
    );
}
#endif

KITE_MODULE_INITIALIZER(_cinvoke_library) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_cinvoke_library");
    kite_set_docstring(newclass, "CInvokeLibrary internal code.");

#ifdef BUILD_CINVOKE    
    /* Add methods. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_compiled_with_docs(thread, CInvokeLibrary_construct, 
                                  "Class constructor.", 2,
                                  "ctx", "CInvokeContext instance",
                                  "lib", "Path to library to load"));
    kite_add_method(thread, newclass, "destruct",
        kite_new_method_compiled_with_docs(thread, CInvokeLibrary_destruct, 
                                  "Free associated memory and resources.", 0));
    kite_add_method(thread, newclass, "get_entrypoint",
        kite_new_method_compiled_with_docs(thread, CInvokeLibrary_entryPoint, 
                                  "Retrieves function pointer for given function.", 1,
                                  "name", "Name of function."));
#endif

    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_cinvoke_library", 1, NULL);
    kite_set_property(thread, parent, "_cinvoke_library", newclass);
}
